
#include "StdAfx.h"
#include "mrs_rpc.h"
#include "mrs_rpc_s.c"

#include "map_2d.h"
#include "diff_wheel_robot.h"
#include "feature_2d.h"
#include "feature_3d.h"
#include "ceiling_landmark.h"
#include "floor_landmark.h"
#include "ceiling_landmark.h"
#include "floor_landmark.h"
#include "scanning_laser.h"
#include "beacon.h"

#include <math.h>
#include "mm.h"

void RegisteryClearRestrictRemoteClients ()
{
	HKEY keyHandle = 0;
	const char *keyStr = "SOFTWARE\\Policies\\Microsoft\\Windows NT\\RPC";
	const char *valueName = "RestrictRemoteClients";
	DWORD value = 0;	// 0 -    , 1 or 2 -     (Windows XP SP2 ⺻ )

	LONG error = RegOpenKeyEx (HKEY_LOCAL_MACHINE, keyStr, 0, KEY_ALL_ACCESS, &keyHandle);
	if (error != ERROR_SUCCESS) {
		DWORD dwDisp;
		error = RegCreateKeyEx(HKEY_LOCAL_MACHINE, keyStr, 0, NULL, REG_OPTION_NON_VOLATILE,
          KEY_WRITE, NULL, &keyHandle, &dwDisp);
	
			//RegCreateKeyEx(HKEY_LOCAL_MACHINE, keyStr, 0, "REG_BINARY", 
			//REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, 0, &keyHandle, &dwDisp);
	}		
	if (error == ERROR_SUCCESS) {
		RegSetValueEx (keyHandle, valueName, 0, REG_DWORD, (BYTE *)&value, sizeof(DWORD)); 
		
		RegCloseKey(keyHandle);	
	}
}

UINT __cdecl rpc_server(LPVOID pParam)
{
	// Uses the protocol combined with the endpoint for receiving remote procedure calls.
	RPC_STATUS status = RpcServerUseProtseqEp(
		(unsigned char*)("ncacn_ip_tcp"),	// Use TCP/IP protocol
		RPC_C_PROTSEQ_MAX_REQS_DEFAULT,		// Backlog q length for TCP/IP.
		(unsigned char*)("9292"),			// TCP/IP port to use.
		NULL);								// No security.

	if(status) return status;

	// Registers the DoRPC interface.
	status = RpcServerRegisterIf(
		MrsRPC_v1_0_s_ifspec,	// Interface to register.
		NULL,					// Use the MIDL generated entry-point vector.
		NULL);					// Use the MIDL generated entry-point vector.

	if (status) return status;

	// Start to listen for remote procedure calls for all registered interfaces.
	// This call will not return until RpcMgmtStopServerListening is called.
	status = RpcServerListen(
		1,								// Recommended minimum number of threads.
		RPC_C_LISTEN_MAX_CALLS_DEFAULT, // Recommended maximum number of threads.
		FALSE);							// Start listening now.

	if (status) return status;

	return 0;
}

// Memory allocation function for RPC.
// The runtime uses these two functions for allocating/deallocating
// enough memory to pass the string to the server.
void* __RPC_USER midl_user_allocate(size_t size)
{
	return malloc(size);
}

// Memory deallocation function for RPC.
void __RPC_USER midl_user_free(void* p)
{
    free(p);
}

#ifdef __cplusplus
extern "C"{
#endif 

void SetMotorAcceleration(double acceleration)
{
	_motor_acceleration = acceleration;
}

void SetMotorMaxVelocity(double maxVelocity)
{
	_motor_max_velocity = maxVelocity;
}

void SetRobotPosition(double x, double y, double theta)
{
	_robot_x = x;
	_robot_y = y;
	_robot_theta = theta;
}

void GetRobotPosition(double *x, double *y, double *theta)
{
	*x = _robot_x;
	*y = _robot_y;
	*theta = _robot_theta;
}

void GetScanningLaserData(double data[ 361 ], long *len)
{
	*len = _laser_scan_count;
	int n = min (_laser_scan_count, 361);
	memcpy (data, _laser_value, n*sizeof(double));
}

void GetFeatureData(long *fid, double *r, double *theta)
{
	double min_r = 1000.;
	long min_id = -1;

	for (unsigned int i=0; i<_feature_2d.size(); ++i) {
		if (_feature_2d[i].active) {
			double dx = _feature_2d[i].x - _robot_x;
			double dy = _feature_2d[i].y - _robot_y;
			double r = sqrt (dx*dx + dy*dy);

			if (r < min_r) {
				min_id = i;
				min_r = r;
			}
		}
	}

	if (0 <= min_id) {
		unsigned int i = min_id;

		double dx = _feature_2d[i].x - _robot_x;
		double dy = _feature_2d[i].y - _robot_y;

		*fid = i;
		*r = sqrt (dx*dx + dy*dy);
		*theta = DeltaRad (atan2 (dy, dx), _robot_theta);
	}
	else {
		*fid = -1;
		*r = 0.;
		*theta = 0.;
	}
}

void GetFeature3DData(long *fid, double *r, double *theta, double *alpha)
{
	double min_r = 1000.;
	long min_id = -1;

	for (unsigned int i=0; i<_feature_3d.size(); ++i) {
		if (_feature_3d[i].active) {
			double dx = _feature_3d[i].x - _robot_x;
			double dy = _feature_3d[i].y - _robot_y;
			double dz = _feature_3d[i].z - 0.;
			double r = sqrt (dx*dx + dy*dy + dz*dz);

			if (r < min_r) {
				min_id = i;
				min_r = r;
			}
		}
	}

	if (0 <= min_id) {
		unsigned int i = min_id;

		double dx = _feature_3d[i].x - _robot_x;
		double dy = _feature_3d[i].y - _robot_y;
		double dz = _feature_3d[i].z - 0.;

		*fid = i;
		*r = sqrt (dx*dx + dy*dy + dz*dz);
		*theta = DeltaRad (atan2 (dy, dx), _robot_theta);
		*alpha = acos (sqrt(dx*dx + dy*dy) / *r);
	}
	else {
		*fid = -1;
		*r = 0.;
		*theta = 0.;
	}
}

void GetCeilingLandmark(long *fid, double *x, double *y, double *theta)
{
	for (unsigned int i=0; i<_ceiling_landmark.size(); ++i) {
		if (_ceiling_landmark[i].active) {
			*fid = _ceiling_landmark[i].id;
			*x = _ceiling_landmark[i].x - _robot_x;
			*y = _ceiling_landmark[i].y - _robot_y;
			*theta = DeltaRad(_ceiling_landmark[i].theta, _robot_theta);
			return;
		}
	}
}

void GetFloorLandmark(unsigned long long fid[10])
{
	int count = 0;
	
	for (map<unsigned long long, sFloorLandmark>::iterator it = _floor_landmark.begin(); it != _floor_landmark.end(); it++) {
		sFloorLandmark &landmark = it->second;
		if (landmark.active) {
			fid[count++] = landmark.uid;
		}
	}
	for (; count < 10; ) {
		fid[count++] = -1;
	}
}

void GetBeacon(long fid[10], double r[10])
{
	int count = 0;
	for (unsigned int i=0; i<_beacon.size() && count < 10; ++i) {
		if (_beacon[i].active) {
			fid[count] = _beacon[i].id;
			r[count++] = _beacon[i].d;
		}
	}
	for (; count < 10; ) {
		fid[count] = -1;
		r[count++] = 0.;
	}
}

void GoMotorVelocity(double leftMotor, double rightMotor)
{
	_target_velocity_left = leftMotor;
	_target_velocity_right= rightMotor;
}

void GetMotorPosition(double *leftMotor, double *rightMotor)
{
	*leftMotor = _current_position_left;
	*rightMotor = _current_position_right;
}

#ifdef __cplusplus
}
#endif
